-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: DPS")

L:RegisterTranslations("enUS", function() return {
	["Damage per Second"] = true,
	["DPS"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Damage per Second"] = "초당 피해량",
	["DPS"] = "DPS",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil

local event = {
	eventType = "Damage",
	sourceID_ne = false,
}

local data = nil
local dirty = nil
local lastUpdateTime = {}
local seconds = nil
local damageDone = nil
local moduleName = "DPS"
local isInCombat = {}

local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName, "Parser-3.0")
mod.revision = tonumber(("$Revision: 35034 $"):sub(12, -3)) or 1
mod.version = 3

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = self:new()
end

function mod:OnModuleEnable()
	self:RegisterParserEvent(event, "OnParserEvent")
	self:RegisterEvent("PLAYER_REGEN_ENABLED") -- handle player dps time via regen enabled
	self:ScheduleRepeatingEvent( self.CheckPartyCombat, 1 )
end

function mod:OnReset()
	if type(data) == "table" then
		data = self:del(data)
	end

	data = self:new()
	data[playerName] = 0
	
	if type(seconds) == "table" then
		seconds = self:del(seconds)
	end
	seconds = self:new()
	
	if type(damageDone) == "table" then
		damageDone = self:del(damageDone)
	end
	damageDone = self:new()
	dirty = nil
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Damage per Second"] end
function mod:GetShortDisplayName() return L["DPS"] end
function mod:GetBroadcastData()
	dirty = nil
	return data[playerName]
end

function mod:IsDirty()
	return dirty
end

function mod:OnRemoteData(sender, input)
	data[sender] = input
end

function mod:PLAYER_REGEN_ENABLED()
	lastUpdateTime[playerName] = nil
end

function mod:OnParserEvent(event)
	if event.amount and event.sourceName then
		local newTime = GetTime()
		local timeSinceLastUpdate
		local name = event.sourceName

		if not lastUpdateTime[name] then
			timeSinceLastUpdate = 0.1
		else
			timeSinceLastUpdate = newTime - lastUpdateTime[name]
		end
		lastUpdateTime[name] = newTime
		seconds[name] = (seconds[name] or 0 ) + timeSinceLastUpdate

		damageDone[event.sourceName] = ( damageDone[event.sourceName] or 0 ) + event.amount		
		
		local newDPS = math.floor(damageDone[event.sourceName] / seconds[name])
		if event.sourceID == "player" then
			dirty = newDPS - (data[playerName] or 0)
			data[playerName] = newDPS
		elseif not Violation:UserViolation( event.sourceName, moduleName ) then			
			self:OnRemoteData( event.sourceName, newDPS ) 
		end
	end
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end

function mod:CheckPartyCombat()
	for name, unit in pairs( roster.roster ) do
		if isInCombat[name] and not UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = nil
			lastUpdateTime[name] = nil
		elseif not isInCombat[name] and UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = true
		end
	end
end
